
//改名卡列表
import styles from './CardList.less'
import React, { Fragment } from 'react'
import { Flex, Result, ActivityIndicator, Toast, Icon } from 'antd-mobile'
import { formatExpireTime } from '../utils/base';
import NativeHook from '@/utils/NativeHook'
import * as Format from '@/utils/Format'
import { formatMessage } from 'umi-plugin-react/locale';


class CardList extends React.Component {
  constructor(props) {
    super(props)
    this.state = {
      list: [],
      loading: false,
      firstLoad: false,
      pagination: {
        pageSize: 20,
        pageIndex: 1,
        total: 0
      },
      firstSaveList: false

    }
  }

  componentDidMount() {
    NativeHook.init()
    this.getPropsList(1)
    this.props.saveCardListRef(this)
  }

  componentWillUnmount() {
    NativeHook.destroy()
    // 移除滚动监听
    if (this.listRef) {
      this.listRef.removeEventListener('scroll', this.bindScroll);
    }
    if (this.timer) {
      clearTimeout(this.timer)
    }
  }

  bindScroll = (event) => {
    //console.log(this.listRef.scrollHeight, this.listRef.scrollTop, this.listRef.clientHeight)
    // 滚动的高度
    const scrollTop = this.listRef.scrollTop
    // 视窗高度
    const clientHeight = this.listRef.clientHeight
    // 页面高度
    const scrollHeight = this.listRef.scrollHeight
    // 距离页面底部的高度
    const height = scrollHeight - scrollTop - clientHeight;
    // 判断距离页面底部的高度 
    if (height <= 500) {
      if (this.state.hasNext) {
        this.getPropsList(this.state.pagination.pageIndex + 1)   //加载下一页
      }
    }
  }

  getPropsList = (pageIndex) => {
    const params = {
      //pageIndex,
      propsType: 102
    }
    if (this.state.loading) return
    this.setState({ loading: true }, () => {
      NativeHook.call.getData({
        api: '/api-app/v1/bag/getPropsList',
        params,
        isDialogShow: false
      }, (res) => {
        if (res.success) {
          const { total, pageSize } = res.data
          let { hasNext, pagination, list } = this.state
          if (pageIndex > 1) {
            list = [...list, ...res.data.list]
          } else {
            list = res.data.list
          }
          if (pageIndex * pagination.pageSize < total) {
            hasNext = true
          } else {
            hasNext = false
          }
          pagination.pageIndex = pageIndex
          pagination.pageSize = pageSize
          const lastFirstLoad = this.state.firstLoad
          if (!this.state.firstSaveList) {
            this.props.saveCardList(list)
          }
          this.setState({ loading: false, list, hasNext, pagination, firstLoad: false, firstSaveList: true })
        } else {
          this.setState({ loading: false, firstLoad: false }, () => {
            Toast.fail(res.msg)
            console.log(res.msg)
          })
        }
      })
    })
  }

  testLog() {
    console.log(this.state)
  }

  handleSwitch(index) {
    const { list } = this.state
    list[index].remarkVisible = !list[index].remarkVisible
    this.setState({ list })
  }

  pickCard(info, disable) {
    if (disable) return
    this.props.pickCard(info)
  }

  //构建靓号剩余时间
  renderLeftTime(expireTime) {
    if (!expireTime) return
    if (expireTime === '-1') {
      return <span>{formatMessage({ id: 'Permanent', defaultMessage: '永久' })}</span>
    } else {
      return <span>{Format.date(expireTime, 'yyyy-MM-dd HH:mm')}</span>
    }
  }

  render() {
    const { loading, err, errMag, firstLoad, list } = this.state
    const { numInfo } = this.props
    return (
      <div className={styles.cardList}>
        {firstLoad && <ActivityIndicator className={styles.firstLoadLoading} toast animating={true} text="加载中" />}
        {
          !firstLoad && err && (
            <Result
              img={<img style={{ width: '1.2rem', height: '1.2rem' }} src={require('@/assets/icon-err.png')} />}
              title={formatMessage({ id: 'Permanent', defaultMessage: '信息错误' })}
              message={errMag}
            />
          )
        }
        {!firstLoad && !err && (
          <Fragment>
            <div className={styles.list} ref={ref => this.listRef = ref}>
              {list.length > 0 && (
                <Fragment>
                  {list.map((item, i) => {
                    const cardInfo = item.item || {}
                    const timeType = item.expireTime === '-1' ? 0 : 1
                    const leftTime = formatExpireTime(timeType, item.expireTime - Date.now()) || {}
                    const disable = cardInfo.propsExtInt === numInfo.niceLevel ? false : true
                    return (
                      <div key={i} className={styles['list-item']}>
                        <Flex justify="between" align="center">
                          <img className={styles['img-niceNum']} src={require('@/assets/niceNum.png')} />
                          <div className={styles.content}>
                            <div className={styles.name}>
                              {cardInfo.propsName}
                              <span style={{ marginLeft: '0.1rem' }} className="font-24 color-999">x{item.quantity}</span>
                            </div>
                            {/* <div>贵族特权</div> */}
                            <div className={styles.time}>{timeType === 0 ? formatMessage({ id: 'Permanent', defaultMessage: '永久' }) : leftTime.days + formatMessage({ id: 'days', defaultMessage: '天' }) + leftTime.hours + formatMessage({ id: 'hours', defaultMessage: '小时' })}</div>
                          </div>
                          <div className={disable ? styles['button-disable'] : styles.button} onClick={this.pickCard.bind(this, item, disable)}>{formatMessage({ id: 'Use', defaultMessage: '使用' })}</div>
                        </Flex>
                        {cardInfo.propsIntro && (
                          <Fragment>
                            <div
                              className={item.remarkVisible ? styles['switch-no'] : styles.switch}
                              onClick={this.handleSwitch.bind(this, i)}><Icon type={item.remarkVisible ? 'up' : 'down'} /></div>
                            <div
                              className={item.remarkVisible ? styles.remark : `${styles.remark} ${styles.noHeight}`}
                            >
                              {cardInfo.propsIntro}
                            </div>
                          </Fragment>
                        )}

                      </div>
                    )
                  })}
                </Fragment>
              )}
              {list.length === 0 && <div className="color-999" style={{ padding: '2rem 0rem' }}>{formatMessage({ id: 'Empty', defaultMessage: '暂无可用靓号卡' })}</div>}
              <div style={{ textAlign: 'center' }}>
                {loading && <ActivityIndicator animating className={styles.loadingMore} />}
              </div>
            </div>
          </Fragment>
        )}
      </div>
    )
  }

}

export default CardList