import React, { Fragment } from 'react'
import { Flex, Modal, Toast, Result, Tabs, ActivityIndicator, Icon } from 'antd-mobile';
import NativeHook from '@/utils/NativeHook'
import Url from '@/utils/Url'
import styles from './Home.less'
import { defaultAvatar } from '@/utils/base'
import { AnchorLevel, PopContainer } from '../components';
import { randomNum } from '../utils/base';
import CardList from './CardList';
import * as Format from '@/utils/Format'
import { formatMessage, getLocale } from 'umi-plugin-react/locale';

class Home extends React.Component {
  constructor(props) {
    super(props)
    this.locale = getLocale()   //国际化语言
    this.state = {
      userInfo: {
        cityInfo: {}
      },
      cardListVisible: false,
      id: Url.getQueryString('id'),
      cardList: [],     //我的靓号卡列表
      cardInfo: {
        item: {}
      },     //靓号卡信息
      numInfo: {},      //靓号信息
    }
  }

  componentDidMount() {
    NativeHook.init()
    document.title = formatMessage({ id: 'home.document.title', defaultMessage: '兑换靓号' })
    this.getProfile()
  }

  componentWillUnmount() {
    NativeHook.destroy()
    if (this.timer) {
      clearTimeout(this.timer)
    }
  }

  getProfile() {
    NativeHook.call.getData({
      api: '/api-app/v1/user/getProfile',
      params: {},
      isDialogShow: false
    }, (res) => {
      if (res.success) {
        const userInfo = res.data
        this.setState({ userInfo })
      }
    })
  }

  //使用靓号卡
  pickCard = (cardInfo) => {
    console.log('选择靓号卡', cardInfo)
    const { numInfo, exchangeType } = this.state
    this.setState({ cardInfo }, () => {
      if (exchangeType === 'renewNiceNo') {
        //续期
        Modal.alert(formatMessage({ id: 'Renew', defaultMessage: '续期靓号' }), formatMessage({ id: 'home.renew.title', defaultMessage: '确定使用"{name}"续期吗？' }, { name: cardInfo.item.propsName }), [
          { text: formatMessage({ id: 'Cancel', defaultMessage: '取消' }), onPress: () => console.log('cancel') },
          { text: formatMessage({ id: 'Confirm', defaultMessage: '确定续期' }), onPress: () => this.renewNiceNo(cardInfo) },
        ])
      } else {
        //兑换靓号
        const propsExtInt = cardInfo.item.propsExtInt
        if (propsExtInt === 1) {
          this.state.tabPaneRef1.showExchange(numInfo)
        }
        if (propsExtInt === 2) {
          this.state.tabPaneRef2.showExchange(numInfo)
        }
        if (propsExtInt === 3) {
          this.state.tabPaneRef3.showExchange(numInfo)
        }
      }

    })
  }

  //续期
  renewNiceNo(cardInfo) {
    NativeHook.call.postData({
      api: '/api-app/v1/props/renewNiceNo',
      params: { bagId: cardInfo.bagId },
      isDialogShow: false
    }, (res) => {
      if (res.success) {
        Toast.success(formatMessage({ id: 'Success', defaultMessage: '续期成功！' }), 2)
        this.setState({ cardListVisible: false })
        this.getProfile()
        if (this.state.cardListRef && this.state.cardListRef.getPropsList) {
          this.state.cardListRef.getPropsList()   //刷新靓号卡列表
        }
      }
    })
  }

  saveCardList = (cardList) => {
    const { id } = this.state
    let { tabIndex } = this.state
    let cardInfo = { item: {} }
    for (let card of cardList) {
      if (card.bagId === id) {
        cardInfo = card
        const propsExtInt = cardInfo.item.propsExtInt
        const needIndex = propsExtInt - 1
        if (needIndex < 3 || needIndex > -1) {
          tabIndex = needIndex
        }
        break
      }
    }

    this.setState({ cardList, cardInfo, tabIndex })
  }

  //兑换成功后，刷新用户信息，刷新靓号卡列表
  refresh = () => {
    this.getProfile()   //刷新用户信息
    if (this.state.cardListRef && this.state.cardListRef.getPropsList) {
      this.state.cardListRef.getPropsList()   //刷新靓号卡列表
    }
  }

  testLog() {
    console.log(this.state)
  }

  handleTabChange = (tab, tabIndex) => {
    this.setState({ tabIndex })
  }

  //打开卡列表
  openCardList = (numInfo, exchangeType) => {
    //type: 'renewNiceNo'靓号续期,    'exchangeNiceNo'兑换
    this.setState({ cardListVisible: true, numInfo, exchangeType })
  }

  closeCardList = () => {
    this.setState({ cardListVisible: false })
  }

  //构建靓号剩余时间
  renderLeftTime() {
    const userNiceNo = this.state.userInfo.userNiceNo || {}
    if (!userNiceNo.bindExpireTime) return
    if (userNiceNo.bindExpireTime === '-1') {
      return <span className="color-white">{formatMessage({ id: 'Permanent', defaultMessage: '永久' })}</span>
    } else {
      return <span className="color-white" >
        {formatMessage({ id: 'home.expires.time', defaultMessage: '靓号将于{time}到期' }, { time: Format.date(userNiceNo.bindExpireTime, 'yyyy-MM-dd HH:mm') })}
      </span>
    }
  }


  render() {
    const { err, errMsg, userInfo, cardListVisible, cardInfo, numInfo } = this.state
    return (
      <div className={styles.niceNum}>
        <div className={styles['card-user']}>
          <div className={styles['card-body']}>
            <img className={styles.avatar} src={userInfo.userAvatar || defaultAvatar} onClick={this.testLog.bind(this)} />
            <Flex align="center" justify="center">
              <div className={styles.name}>{userInfo.userName}</div>
              <div className={styles.level}><AnchorLevel level={userInfo.wealthLevel || 1} /></div>
            </Flex>
            {userInfo.userNiceNo && (
              <Fragment>
                <div className={styles['bar-num']}>
                  <img className={styles['icon-dia']} src={require('@/assets/icon_dia.png')} />
                  {formatMessage({ id: 'home.Vanity.No', defaultMessage: '靓号：{num}' }, { num: userInfo.userNiceNo.niceNo })}
                </div>
                <div style={{ marginTop: '0.2rem' }}>
                  <span>{this.renderLeftTime()}</span>
                  <span style={{ color: '#FCEB9F', marginLeft: '0.1rem' }} onClick={this.openCardList.bind(this, userInfo.userNiceNo, 'renewNiceNo')}>
                    {formatMessage({ id: 'Renew', defaultMessage: '续期' })} >
                  </span>
                </div>
              </Fragment>
            )}
          </div>
        </div>
        {/* tab */}
        <div className={this.locale === 'en-US' ? styles['tab-en'] : styles.tab}>
          <Tabs
            tabs={[{ title: formatMessage({ id: 'Classic', defaultMessage: '经典靓号' }) }, { title: formatMessage({ id: 'Epic', defaultMessage: '史诗靓号' }) }, { title: formatMessage({ id: 'Legand', defaultMessage: '传奇靓号' }) }]}
            initialPage={0}
            swipeable={false}
            page={this.state.tabIndex}
            onChange={this.handleTabChange}
          >
            <TabPane
              id={0} niceLevel={1} cardInfo={cardInfo}
              refresh={this.refresh}
              openCardList={this.openCardList}
              saveTabPaneRef={tabPaneRef1 => this.setState({ tabPaneRef1 })}
              closeCardList={this.closeCardList}
            />
            <TabPane
              id={1} niceLevel={2} cardInfo={cardInfo} refresh={this.refresh}
              openCardList={this.openCardList}
              saveTabPaneRef={tabPaneRef2 => this.setState({ tabPaneRef2 })}
              closeCardList={this.closeCardList}
            />
            <TabPane
              id={2} niceLevel={3} cardInfo={cardInfo}
              refresh={this.refresh}
              openCardList={this.openCardList}
              saveTabPaneRef={tabPaneRef3 => this.setState({ tabPaneRef3 })}
              closeCardList={this.closeCardList}
            />
          </Tabs>
        </div>
        {/* 卡片列表 */}
        <PopContainer visible={cardListVisible} onClose={() => this.setState({ cardListVisible: false })}>
          <CardList
            numInfo={numInfo}
            pickCard={this.pickCard}
            saveCardList={this.saveCardList}
            saveCardListRef={cardListRef => this.setState({ cardListRef })}
          />
        </PopContainer>
      </div>
    )
  }
}

export default Home

class TabPane extends React.Component {
  constructor(props) {
    super(props)
    this.state = {
      loading: false,
      firstLoad: false,
      list: [],
      defaultList: []
    }
  }

  componentDidMount() {
    NativeHook.init()
    this.getNiceNoList()
    this.props.saveTabPaneRef(this)
  }

  componentWillUnmount() {
    NativeHook.destroy()
    if (this.timer) {
      clearTimeout(this.timer)
    }
  }

  //获取靓号列表
  getNiceNoList() {
    NativeHook.call.getData({
      api: '/api-app/v1/props/getNiceNoList',
      params: { niceLevel: this.props.niceLevel },
      isDialogShow: false
    }, (res) => {
      if (res.success) {
        const defaultList = res.data.list || []
        let list = []
        if (defaultList.length > 9) {
          const index = randomNum(0, defaultList.length - 9)
          list = defaultList.slice(index, index + 9)
          console.log(index, list)
        } else {
          list = defaultList
        }
        this.setState({ defaultList, list })

      }
    })
  }

  onChange() {
    if (this.state.loading) return
    this.setState({ loading: true }, () => {
      this.timer = setTimeout(() => {
        const { defaultList } = this.state
        let list = []
        if (defaultList.length > 9) {
          const index = randomNum(0, defaultList.length - 9)
          list = defaultList.slice(index, index + 9)
          console.log(index, list)
        } else {
          list = defaultList
        }
        this.setState({ list, loading: false })
      }, 500);
    })
  }

  //兑换靓号
  exchangeNiceNo(numInfo) {
    const { cardInfo } = this.props
    const { list, defaultList, exchangeLoading } = this.state
    const params = {
      bagId: cardInfo.bagId,
      niceNo: numInfo.niceNo
    }
    if (exchangeLoading) return
    this.setState({ exchangeLoading: true }, () => {
      NativeHook.call.postData({
        api: '/api-app/v1/props/exchangeNiceNo',
        params,
        isDialogShow: false
      }, (res) => {
        if (res.success) {
          Toast.success(formatMessage({ id: 'Success', defaultMessage: '兑换成功' }), 1.5)
          this.props.refresh()
          this.getNiceNoList()    //刷新卡池
          this.setState({ list, defaultList, exchangeLoading: false })
          this.props.closeCardList()    //关闭列表
        } else {
          Toast.fail(res.msg, 2)
          this.setState({ exchangeLoading: false })
        }
      })
    })

  }

  showExchange = (numInfo) => {
    const { cardInfo } = this.props
    Modal.alert(formatMessage({ id: 'Exchange Vanity No', defaultMessage: '兑换靓号' }), formatMessage({ id: 'Exchange Vanity No', defaultMessage: '确定使用"{name}"兑换吗？已有的靓号将被替换!' }, { name: cardInfo.item.propsName }), [
      { text: formatMessage({ id: 'Cancel', defaultMessage: '取消' }), onPress: () => console.log('cancel') },
      { text: formatMessage({ id: 'Confirm', defaultMessage: '确定兑换' }), onPress: () => this.exchangeNiceNo(numInfo) },
    ])
  }

  //打开卡列表
  openCardList(numInfo) {
    this.props.openCardList(numInfo)
  }


  render() {
    const { loading, firstLoad, err, errMag, list } = this.state
    return (
      <div className={`${styles.tabPane} ${styles[`tabPane_${this.props.id}`]}`}>
        {firstLoad && <ActivityIndicator toast animating={true} text={formatMessage({ id: 'Loading', defaultMessage: '加载中' })} />}
        {
          !firstLoad && err && (
            <Result
              img={<img style={{ width: '1.2rem', height: '1.2rem' }} src={require('@/assets/icon-err.png')} />}
              title={formatMessage({ id: 'err', defaultMessage: '信息错误' })}
              message={errMag}
            />
          )
        }
        {!firstLoad && !err && (
          <div className={styles.content}>
            <div style={{ height: '4.6rem' }}>
              {list.length > 0 && (
                <Flex align="start" wrap="wrap" className={styles.list}>
                  {list.map((item, i) => {
                    const noMarginRight = (i + 1) % 3 === 0
                    return (
                      <Flex direction="column" justify="between" align="center" key={i} className={styles['card-num']} style={{ marginRight: noMarginRight ? '0rem' : '0.08rem' }}>
                        <div className={styles.num}>{item.niceNo}</div>
                        <div className={styles.button} onClick={this.openCardList.bind(this, item, 'exchangeNiceNo')}>{formatMessage({ id: 'Exchange', defaultMessage: '立即兑换' })}</div>
                      </Flex>
                    )
                  })}
                </Flex>
              )}
              {list.length === 0 && <div style={{ color: '#ccc', padding: '2rem 0rem' }}>{formatMessage({ id: 'Empty', defaultMessage: '暂无靓号' })}</div>}
            </div>
            <div className={styles['button-change']} onClick={this.onChange.bind(this)}>
              {this.state.loading ? <Icon type="loading" style={{ verticalAlign: 'middle' }} /> : formatMessage({ id: 'Next', defaultMessage: '换一批' })}
            </div>
          </div>
        )}
      </div>
    )
  }
}